@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2002 Jean-Baptiste Mardelle <bj@altern.org>
- * Copyright (C) 2007,2008,2009,2010,2011 Rolf Eike Beer <kde@opensource.sf-tec.de>
+ * Copyright (C) 2007,2008,2009,2010,2011,2012
+ *               Rolf Eike Beer <kde@opensource.sf-tec.de>
  * Copyright (C) 2011 Luis Ángel Fernández Fernández <laffdez@gmail.com>
  */
 
@@ -15,101 +16,99 @@
 
 #include "keysmanager.h"
 
-#include <QApplication>
-#include <QDesktopWidget>
-#include <QKeySequence>
-#include <QTextStream>
-#include <QClipboard>
-#include <QPainter>
-#include <QPrinter>
-#include <QPrintDialog>
-#include <QWidget>
-#include <QLabel>
-#include <QEvent>
-#include <QList>
-#include <QFile>
-#include <QDir>
-#include <QtDBus/QtDBus>
-#include <QProcess>
-
-#include <kabc/addresseedialog.h>
-#include <kabc/stdaddressbook.h>
-#include <solid/networking.h>
-#include <KToolInvocation>
-#include <KStandardDirs>
-#include <KInputDialog>
-#include <KFileDialog>
-#include <KMessageBox>
-#include <KFindDialog>
-#include <KService>
-#include <KMimeTypeTrader>
-#include <KLineEdit>
-#include <KMimeType>
-#include <KRecentFilesAction>
-#include <KShortcut>
-#include <KStandardShortcut>
-#include <KLocale>
-#include <KProcess>
-#include <KAction>
-#include <KDebug>
-#include <KFind>
-#include <KMenu>
-#include <KUniqueApplication>
-#include <KUrl>
-#include <ktip.h>
-#include <KActionCollection>
-#include <KStandardAction>
-#include <KSelectAction>
-#include <KIcon>
-#include <KVBox>
-#include <KToggleAction>
-#include <KStatusBar>
-#include <KToolBar>
-#include <kio/global.h>
-#include <kjobtrackerinterface.h>
-
 #include "caff.h"
-#include "kgpgkey.h"
-#include "selectsecretkey.h"
-#include "newkey.h"
-#include "kgpg.h"
-#include "kgpgeditor.h"
+#include "core/images.h"
+#include "core/kgpgkey.h"
+#include "detailedconsole.h"
+#include "groupedit.h"
+#include "keyadaptor.h"
 #include "keyexport.h"
-#include "kgpgrevokewidget.h"
+#include "keyinfodialog.h"
 #include "keyservers.h"
-#include "kgpginterface.h"
-#include "kgpgsettings.h"
+#include "keytreeview.h"
+#include "kgpg.h"
+#include "kgpgchangekey.h"
 #include "kgpgkeygenerate.h"
 #include "kgpgoptions.h"
-#include "keyinfodialog.h"
-#include "keyadaptor.h"
-#include "images.h"
-#include "sourceselect.h"
-#include "keylistproxymodel.h"
-#include "keytreeview.h"
-#include "groupedit.h"
-#include "kgpgchangekey.h"
-#include "kgpgdeluid.h"
-#include "kgpgaddphoto.h"
-#include "kgpgadduid.h"
-#include "kgpgexport.h"
-#include "kgpggeneratekey.h"
-#include "kgpggeneraterevoke.h"
-#include "kgpgdelkey.h"
-#include "kgpgimport.h"
-#include "kgpgprimaryuid.h"
-#include "detailedconsole.h"
-#include "kstatusnotifieritem.h"
+#include "kgpgrevokewidget.h"
+#include "kgpgsettings.h"
+#include "newkey.h"
 #include "selectpublickeydialog.h"
-#include "kgpgtextinterface.h"
-#include "kgpgtextedit.h"
-#include "kgpgkeyservergettransaction.h"
-#include "kgpgtransactionjob.h"
-#include "kgpgsignkey.h"
-#include "kgpgsignuid.h"
-#include "kgpgdelsign.h"
-#include "kgpgdecrypt.h"
-#include "kgpgencrypt.h"
+#include "selectsecretkey.h"
+#include "sourceselect.h"
+#include "editor/kgpgeditor.h"
+#include "editor/kgpgtextedit.h"
+#include "model/keylistproxymodel.h"
+#include "transactions/kgpgaddphoto.h"
+#include "transactions/kgpgadduid.h"
+#include "transactions/kgpgdecrypt.h"
+#include "transactions/kgpgdelkey.h"
+#include "transactions/kgpgdelsign.h"
+#include "transactions/kgpgdeluid.h"
+#include "transactions/kgpgencrypt.h"
+#include "transactions/kgpgexport.h"
+#include "transactions/kgpggeneratekey.h"
+#include "transactions/kgpggeneraterevoke.h"
+#include "transactions/kgpgimport.h"
+#include "transactions/kgpgkeyservergettransaction.h"
+#include "transactions/kgpgprimaryuid.h"
+#include "transactions/kgpgsignkey.h"
+#include "transactions/kgpgsignuid.h"
+#include "transactions/kgpgtransactionjob.h"
+
+#include <KAction>
+#include <KActionCollection>
+#include <KDebug>
+#include <KFileDialog>
+#include <KFind>
+#include <KFindDialog>
+#include <KIcon>
+#include <KInputDialog>
+#include <KLineEdit>
+#include <KLocale>
+#include <KMenu>
+#include <KMessageBox>
+#include <KMimeType>
+#include <KMimeTypeTrader>
+#include <KProcess>
+#include <KRecentFilesAction>
+#include <KSelectAction>
+#include <KService>
+#include <KShortcut>
+#include <KStandardAction>
+#include <KStandardDirs>
+#include <KStandardShortcut>
+#include <KStatusBar>
+#include <KStatusNotifierItem>
+#include <KToggleAction>
+#include <KToolBar>
+#include <KToolInvocation>
+#include <KUniqueApplication>
+#include <KUrl>
+#include <KVBox>
+#include <QApplication>
+#include <QClipboard>
+#include <QDesktopWidget>
+#include <QDir>
+#include <QEvent>
+#include <QFile>
+#include <QKeySequence>
+#include <QLabel>
+#include <QList>
+#include <QMetaObject>
+#include <QPainter>
+#include <QPrintDialog>
+#include <QPrinter>
+#include <QProcess>
+#include <QTextStream>
+#include <QWidget>
+#include <QtDBus/QtDBus>
+#include <kabc/addresseedialog.h>
+#include <kabc/stdaddressbook.h>
+#include <kio/global.h>
+#include <kjobtrackerinterface.h>
+#include <ktip.h>
+#include <solid/networking.h>
 
 using namespace KgpgCore;
 
@@ -169,11 +168,6 @@ KeysManager::KeysManager(QWidget *parent)
 	action->setText(i18n("&Refresh List"));
 	action->setShortcuts(KStandardShortcut::reload());
 
-	hPublic = actionCollection()->add<KToggleAction>(QLatin1String("show_secret"), this, SLOT(slotToggleSecret(bool)));
-	hPublic->setIcon(KIcon( QLatin1String( "view-key-secret" )));
-	hPublic->setText(i18n("&Show Only Secret Keys"));
-	hPublic->setChecked(KGpgSettings::showSecret());
-
 	longId = actionCollection()->add<KToggleAction>(QLatin1String("show_long_keyid"), this, SLOT(slotShowLongId(bool)));
 	longId->setText(i18n("Show &Long Key Id"));
 	longId->setChecked(KGpgSettings::showLongKeyId());
@@ -336,6 +330,11 @@ KeysManager::KeysManager(QWidget *parent)
 
 	connect (iview, SIGNAL(returnPressed()), SLOT(slotDefaultAction()));
 
+	hPublic = actionCollection()->add<KToggleAction>(QLatin1String("show_secret"), iproxy, SLOT(setOnlySecret(bool)));
+	hPublic->setIcon(KIcon( QLatin1String( "view-key-secret" )));
+	hPublic->setText(i18n("&Show Only Secret Keys"));
+	hPublic->setChecked(KGpgSettings::showSecret());
+
 	int psize = KGpgSettings::photoProperties();
 	photoProps->setCurrentItem(psize);
 	slotSetPhotoSize(psize);
@@ -466,7 +465,7 @@ KeysManager::KeysManager(QWidget *parent)
 
 	stateChanged("empty_list");
 
-	QTimer::singleShot(0, this, SLOT(refreshkey()));
+	QMetaObject::invokeMethod(this, "refreshkey", Qt::QueuedConnection);
 }
 
 KeysManager::~KeysManager()
@@ -667,11 +666,6 @@ void KeysManager::slotShowCreation()
 		iview->resizeColumnToContents(KEYCOLUMN_CREAT);
 }
 
-void KeysManager::slotToggleSecret(bool b)
-{
-	iproxy->setOnlySecret(b);
-}
-
 void KeysManager::slotShowLongId(bool b)
 {
 	iproxy->setIdLength(b ? 16 : 8);
@@ -773,8 +767,8 @@ void KeysManager::slotKeyRefreshDone(int result)
 		imodel->refreshKeys(keys);
 
 	QApplication::restoreOverrideCursor();
-	(void) new KgpgDetailedInfo(this, message.join(QLatin1String("\n")), log.join(QLatin1String("\n")),
-			KGpgImport::getDetailedImportMessage(log).split(QLatin1Char( '\n' )));
+	(void) new KgpgDetailedInfo(this, message.join(QLatin1String("\n")), log.join(QLatin1String("<br/>")),
+			KGpgImport::getDetailedImportMessage(log, imodel).split(QLatin1Char('\n')));
 }
 
 void KeysManager::slotDelUid()
@@ -1103,9 +1097,6 @@ void KeysManager::readOptions()
 	if (KGpgSettings::useMouseSelection() && (kapp->clipboard()->supportsSelection()))
 		m_clipboardmode = QClipboard::Selection;
 
-	// re-read groups in case the config file location was changed
-	const QStringList groups(KgpgInterface::getGpgGroupNames(KGpgSettings::gpgConfigPath()));
-	KGpgSettings::setGroups(groups.join( QLatin1String( "," )));
 	if (imodel != NULL)
 		updateStatusCounter();
 
@@ -1631,11 +1622,9 @@ void KeysManager::deleteGroup()
 	if (result != KMessageBox::Continue)
 		return;
 
-	KgpgInterface::delGpgGroup(nd->getName(), KGpgSettings::gpgConfigPath());
+	nd->toGroupNode()->remove();
 	imodel->delNode(nd);
 
-	const QStringList groups(KgpgInterface::getGpgGroupNames(KGpgSettings::gpgConfigPath()));
-	KGpgSettings::setGroups(groups.join( QLatin1String( "," )));
 	updateStatusCounter();
 }
 
@@ -1652,7 +1641,6 @@ void KeysManager::renameGroup()
 void KeysManager::createNewGroup()
 {
 	QStringList badkeys;
-	QStringList keysGroup;
 	KGpgKeyNode::List keysList;
 	KgpgItemType tp;
 	KGpgNode::List ndlist(iview->selectedNodes(NULL, &tp));
@@ -1673,7 +1661,6 @@ void KeysManager::createNewGroup()
 
 	foreach (KGpgNode *nd, ndlist) {
 		if (nd->getTrust() >= mintrust) {
-			keysGroup += nd->getId();
 			keysList.append(nd->toKeyNode());
 		} else {
 			badkeys += i18nc("<Name> (<Email>) ID: <KeyId>", "%1 (%2) ID: %3",
@@ -1686,13 +1673,10 @@ void KeysManager::createNewGroup()
 			QString(), 0, this));
 	if (groupName.isEmpty())
 		return;
-	if (!keysGroup.isEmpty()) {
+	if (!keysList.isEmpty()) {
 		if (!badkeys.isEmpty())
 			KMessageBox::informationList(this, i18n("Following keys are not valid or not trusted and will not be added to the group:"), badkeys);
 
-		KgpgInterface::setGpgGroupSetting(groupName, keysGroup, KGpgSettings::gpgConfigPath());
-		const QStringList groups(KgpgInterface::getGpgGroupNames(KGpgSettings::gpgConfigPath()));
-
 		iview->selectNode(imodel->addGroup(groupName, keysList));
 		updateStatusCounter();
         } else {
@@ -1722,15 +1706,10 @@ void KeysManager::editGroup()
 	dialogGroupEdit->setMainWidget(gEdit);
 
 	gEdit->show();
-	if (dialogGroupEdit->exec() == QDialog::Accepted) {
-		QStringList memberids;
 
-		foreach (KGpgNode *knd, members)
-			memberids << knd->getId();
-
-		KgpgInterface::setGpgGroupSetting(nd->getName(), memberids, KGpgSettings::gpgConfigPath());
+	if (dialogGroupEdit->exec() == QDialog::Accepted)
 		imodel->changeGroup(gnd, members);
-	}
+
 	delete dialogGroupEdit;
 }
 
@@ -2228,49 +2207,43 @@ void KeysManager::doPrint(const QString &txt)
 
 void KeysManager::removeFromGroups(KGpgKeyNode *node)
 {
-	const QList<KGpgGroupNode *> groups(node->getGroups());
-	if (groups.count() == 0)
-		return;
-
 	QStringList groupNames;
 
-	foreach (const KGpgGroupNode *gnd, groups)
+	foreach (const KGpgGroupNode *gnd, node->getGroups())
 		groupNames << gnd->getName();
 
-	QString ask = i18np("<qt>The key you are deleting is a member of the following key group. Do you want to remove it from this group?</qt>",
+	if (groupNames.isEmpty())
+		return;
+
+	const QString ask = i18np("<qt>The key you are deleting is a member of the following key group. Do you want to remove it from this group?</qt>",
 			"<qt>The key you are deleting is a member of the following key groups. Do you want to remove it from these groups?</qt>",
 			groupNames.count());
 
 	if (KMessageBox::questionYesNoList(this, ask, groupNames, i18n("Delete key")) != KMessageBox::Yes)
 		return;
 
-	QList<KGpgGroupMemberNode *> grefs(node->getGroupRefs());
-
 	bool groupDeleted = false;
 
-	foreach (KGpgGroupMemberNode *gref, grefs) {
-		QStringList memberids;
+	foreach (KGpgGroupMemberNode *gref, node->getGroupRefs()) {
 		KGpgGroupNode *group = gref->getParentKeyNode();
 
-		for (int j = group->getChildCount() - 1; j >= 0; j--) {
-			KGpgGroupMemberNode *rn = group->getChild(j)->toGroupMemberNode();
-			if (rn != gref)
-				memberids << rn->getId();
-		}
+		bool deleteWholeGroup = (group->getChildCount() == 1) &&
+				(group->getChild(0)->toGroupMemberNode() == gref);
+		if (deleteWholeGroup)
+			deleteWholeGroup = (KMessageBox::questionYesNo(this,
+					i18n("You are removing the last key from key group %1.<br/>Do you want to delete the group, too?"),
+					i18n("Delete key")) == KMessageBox::Yes);
 
-		if (!memberids.isEmpty()) {
-			KgpgInterface::setGpgGroupSetting(group->getName(), memberids, KGpgSettings::gpgConfigPath());
+		if (!deleteWholeGroup) {
 			imodel->deleteFromGroup(group, gref);
 		} else {
-			KgpgInterface::delGpgGroup(group->getName(), KGpgSettings::gpgConfigPath());
+			group->remove();
 			imodel->delNode(group);
 			groupDeleted = true;
 		}
 	}
 
 	if (groupDeleted) {
-		QStringList groupnames = KgpgInterface::getGpgGroupNames(KGpgSettings::gpgConfigPath());
-		KGpgSettings::setGroups(groupnames.join(QLatin1String(",")));
 		updateStatusCounter();
 	}
 }
